
----------------------------------
-- Create plugin
local plugin = RW:Plugin("VersionDisplay")
if not plugin then return end

----------------------------------
-- Locals 
local AceGUI = LibStub("AceGUI-3.0")
local LSM = LibStub("LibSharedMedia-3.0")
local UIF
local party
local partyNames = {}
local guildNames = {}
local selectedSortMethod
local displayGroup
local updateSchedule
local listFontSize = 15

local sortmethods = {
	["name"] = "By Name",
	["version"] = "By Version",
}

local defaults = {
	profile = {
		PluginEnabled = true,
	}
}

----------------------------------
-- Init

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace("VersionDisplay", defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Reset")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")	
end

function plugin:PluginEnable()
	if not db.PluginEnabled then
		self:Disable()
		return
	end
	
	UIF = LibStub("AceGUI-Factory-1.0"):GetFactory("RW")
end

function plugin:PluginDisable()
end

function plugin:Reset()
	db = self.db.profile
end

local function sortByName(a, b)
	return a.name < b.name
end

local function sortByVersion(a, b)
	if (a and a.version) and (b and b.version) then
		if a.version == b.version then
			if a.revision == b.revision then
				if a.releaseType == b.releaseType then
					return a.name < b.name
				else
					return tostring(a.releaseType) < tostring(b.releaseType)
				end
			else
				return tonumber(a.revision) or 999999 > tonumber(b.revision) or 999999
			end
		else
			return a.version > b.version
		end
	elseif (a and a.version) and (b and not b.version) then
		return true
	elseif (a and not a.version) and (b and  b.version) then
		return false
	else
		return (a and a.name or "") < (b and b.name or "")
	end
end

local function selectedSortMethod(a, b)
	if plugin.db.profile.VersionSort == "name" then
		return sortByName(a, b)
	else
		return sortByVersion(a, b)
	end
end

local function selectSortMethod(widget, event, method)
	plugin.db.profile.VersionSort = method
	plugin:GetGUI(true)
end

local function updateParty()
	party = party or RW:GetPlugin("Party")
	wipe(partyNames)
	if party:GetPartyType() ~= "none" then
		
		for name, data in party:PartyIterator() do
			tinsert(partyNames, data)
		end
		table.sort(partyNames, selectedSortMethod)
	end
end

local function updateGuild()
	wipe(guildNames)
	if RW.GuildVersions then
		for name, data in pairs(RW.GuildVersions) do
			tinsert(guildNames, data)
		end
		table.sort(guildNames, selectedSortMethod)
	end
end

local function addNames(listType, list, group)
	local sgroup, co
	
	local sgroup = UIF:InlineGroup2(listType)
	group:AddChild(sgroup)
	
	col = UIF:Level1Title("Name", nil, listFontSize + 3)
	col:SetRelativeWidth(0.25)
	sgroup:AddChild(col)
	
	col = UIF:Level1Title("Version", nil, listFontSize + 3)
	col:SetRelativeWidth(0.25)
	sgroup:AddChild(col)
	
	col = UIF:Level1Title("Revision", nil, listFontSize + 3)
	col:SetRelativeWidth(0.25)
	sgroup:AddChild(col)
	
	col = UIF:Level1Title("Type", nil, listFontSize + 3)
	col:SetRelativeWidth(0.25)
	sgroup:AddChild(col)
	
	local color1 = {1, 0.6, 0.16}
	local color2 = {1, 1, 1}
	
	for i, data in pairs(list) do
		local color
		if i % 2 == 0 then
			color = color1
		else
			color = color2
		end
	
		col = UIF:Level1Title(data.name, color, listFontSize)
		col:SetRelativeWidth(0.25)
		sgroup:AddChild(col)
		
		col = UIF:Level1Title(data.version or "---", color, listFontSize)
		col:SetRelativeWidth(0.25)
		sgroup:AddChild(col)
		
		col = UIF:Level1Title(data.revision or "---", color, listFontSize)
		col:SetRelativeWidth(0.25)
		sgroup:AddChild(col)
		
		col = UIF:Level1Title(data.releaseType or "---", color, listFontSize)
		col:SetRelativeWidth(0.25)
		sgroup:AddChild(col)
	end
end

function plugin:GetOptions()
	return 8, "Versions", self.GetGUI
end

function plugin:GetGUI(used)
	local self = self or plugin
	
	local group, label, slider, check, igroup, button, color, select
	
	if used then 
		group = displayGroup
		group:ReleaseChildren()
	else
		group = AceGUI:Create("ScrollFrame")
		group:SetLayout("Flow")
		group:SetUserData("used", true)
		displayGroup = group
	end
	
	label = UIF:MainTitleLabel("Versions")
	group:AddChild(label)
	
	igroup = UIF:InlineGroup("")
	group:AddChild(igroup)
	
	select = UIF:Dropdown()
	select:SetCallback("OnValueChanged", selectSortMethod)
	select:SetList(sortmethods)
	select:SetValue(self.db.profile.VersionSort)
	select:SetLabel("Sorting")
	igroup:AddChild(select)
	
	button = UIF:Button("Refresh", function() 
		plugin:GetGUI(true)
	end)
	igroup:AddChild(button)
	
	label = UIF:NormalText(" ")
	igroup:AddChild(label)
	
	updateParty()
	if #partyNames > 0 then
		addNames("Party", partyNames, igroup)		
	end
	
	updateGuild()
	if #guildNames > 0 then
		addNames("Guild", guildNames, igroup)	
	end
	
	group:FixScroll()
	
	return group
end